<?php

namespace Drupal\frontendpublishing\Plugin\rest\resource;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\frontendpublishing\Service\MenuHelper;
use Drupal\frontendpublishing\Service\StateChange;
use Drupal\rest\ModifiedResourceResponse;
use Symfony\Component\HttpFoundation\Request;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a resource to move a page in the menu tree.
 *
 * @RestResource(
 *   id = "frontendpublishing_move",
 *   label = @Translation("Move a page"),
 *   uri_paths = {
 *     "canonical" = "/frontendpublishing/move"
 *   }
 * )
 */
class Move extends ResourceBase {
  /**
   * The state change service.
   *
   * @var \Drupal\frontendpublishing\Service\StateChange
   */
  protected $stateChange = NULL;

  /**
   * The menu helper service.
   *
   * @var \Drupal\frontendpublishing\Service\MenuHelper
   */
  protected $menuHelper = NULL;

  /**
   * Constructs a new UnpublishResource object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   * @param \Drupal\frontendpublishing\Service\StateChange $state_change
   *   State Change Service.
   * @param \Drupal\frontendpublishing\Service\MenuHelper $menu_helper
   *   Menu Helper Service.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              array $serializer_formats,
                              LoggerInterface $logger,
                              AccountProxyInterface $current_user,
                              StateChange $state_change,
                              MenuHelper $menu_helper) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->stateChange = $state_change;
    $this->menuHelper = $menu_helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('frontendpublishing'),
      $container->get('current_user'),
      $container->get('frontendpublishing.state_change'),
      $container->get('frontendpublishing.menu_helper')
    );
  }

  /**
   * Moves a page under the new parent at the given position. Rebuilds menu.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Drupal\rest\ModifiedResourceResponseModifiedResourceResponse
   *   The response containing a list of bundle names.
   */
  public function patch(Request $request) {
    $requestContent = json_decode($request->getContent(), TRUE);
    $pluginId = $requestContent['id'];
    $parentId = $requestContent['newParent'];
    $position = $requestContent['weight'];
    $menu = $requestContent['menu'];

    $response = [];
    if (empty($menu) || $menu == NULL) {
      $menu = 'main';
    }

    $this->_menuHelper = \Drupal::service('frontendpublishing.menu_helper');
    $menuLink = $this->_menuHelper::loadMenuLink($pluginId);
    if ($menuLink != NULL) {
      $oldParent = $menuLink->parent->value;
      if (!empty($parentId)) {
        $parentMenuLink = $this->_menuHelper::loadMenuLink($parentId);
        if ($parentMenuLink != NULL) {
          $menuLink->parent->value = $parentMenuLink->getPluginId();
        }
        else {
          $menuLink->parent->value = '';
        }
      }
      else {
        $menuLink->parent->value = '';
      }

      if ($menuLink->parent->value != $oldParent) {
        $this->_menuHelper::reorder($oldParent, NULL, $menu);
      }
      $this->_menuHelper::saveAndReorder($menuLink, $position, $menu);
      $this->_menuHelper::clearCache($menu);

      $response = $requestContent;
    }
    else {
      $response = ['error' => 'Could not find menu link ' . $pluginId . ' in menu ' . $menu];
    }
    return new ModifiedResourceResponse($response);
  }

}
